@@ -43,7 +43,6 @@ def load_config(config):
 
 def update_available():
     urllib3.disable_warnings()
-    logger.info("Checking for updates...")
     if "b" not in __version__:
         version_request = "https://raw.githubusercontent.com/GramAddict/bot/master/GramAddict/version.py"
     else:
@@ -77,10 +76,42 @@ def update_available():
     return False, online_version_raw
 
 
+def check_if_updated(crash=False):
+    if not crash:
+        logger.info("Checking for updates...")
+    new_update, version = update_available()
+    if new_update:
+        logger.warning("NEW VERSION FOUND!")
+        logger.warning(
+            f"Version {version} has been released! Please update so that you can get all the latest features and bugfixes. Changelog here -> https://github.com/GramAddict/bot/blob/master/CHANGELOG.md"
+        )
+        logger.warning("HOW TO UPDATE:")
+        logger.warning("If you installed with pip: pip3 install GramAddict -U")
+        logger.warning("If you installed with git: git pull")
+        sleep(5)
+    else:
+        if not crash:
+            logger.info("Bot is updated.", extra={"color": f"{Style.BRIGHT}"})
+    if not crash:
+        logger.info(
+            f"GramAddict v.{__version__}",
+            extra={"color": f"{Style.BRIGHT}{Fore.MAGENTA}"},
+        )
+
+
+def ask_for_a_donation():
+    logger.info(
+        "This bot is backed with love by me for free. If you like using it, consider donating to help keep me motivated: https://www.buymeacoffee.com/mastrolube",
+        extra={"color": f"{Style.BRIGHT}{Fore.MAGENTA}"},
+    )
+
+
 def move_usernames_to_accounts():
     Path(ACCOUNTS).mkdir(parents=True, exist_ok=True)
     ls = next(walk("."))[1]
     ignored_dir = [
+        "__pycache__",
+        "build",
         "accounts",
         "GramAddict",
         "config-examples",
@@ -280,6 +311,16 @@ def close_instagram(device, screen_record):
             )
 
 
+def print_telegram_reports(
+    configs, telegram_reports_at_end, followers_now, following_now, time_left=None
+):
+    if followers_now is not None:
+        if telegram_reports_at_end:
+            configs.actions["telegram-reports"].run(
+                configs, "telegram-reports", followers_now, following_now, time_left
+            )
+
+
 def kill_atx_agent(device):
     logger.debug("Back to default keyboard!")
     device.deviceV2.set_fastinput_ime(False)
@@ -344,6 +385,7 @@ def save_crash(device):
         extra={"color": Fore.GREEN},
     )
     logger.info("https://discord.gg/66zWWCDM7x\n", extra={"color": Fore.GREEN})
+    check_if_updated(crash=True)
 
 
 def stop_bot(device, sessions, session_state, screen_record, was_sleeping=False):
@@ -357,6 +399,7 @@ def stop_bot(device, sessions, session_state, screen_record, was_sleeping=False)
         print_full_report(sessions, configs.args.scrape_to_file)
         if not was_sleeping:
             sessions.persist(directory=session_state.my_username)
+    ask_for_a_donation()
     sys.exit(0)
 
 
@@ -378,7 +421,7 @@ def can_repeat(current_session, max_sessions):
         return True
 
 
-def get_value(count, name, default):
+def get_value(count, name, default, its_time=False):
     def print_error():
         logger.error(
             name.format(default)
@@ -400,7 +443,11 @@ def get_value(count, name, default):
             print_error()
     elif len(parts) == 2:
         try:
-            value = randint(int(parts[0]), int(parts[1]))
+            if not its_time:
+                value = randint(int(parts[0]), int(parts[1]))
+            else:
+                value = round(uniform(int(parts[0]), int(parts[1])), 2)
+
             if name is not None:
                 logger.info(name.format(value), extra={"color": Style.BRIGHT})
         except ValueError:
@@ -425,7 +472,7 @@ def append_to_file(filename, username):
     try:
         if not filename.lower().endswith(".txt"):
             filename = filename + ".txt"
-        with open(filename, "a+", encoding="UTF-8") as file:
+        with open(filename, "a+", encoding="utf-8") as file:
             file.write(username + "\n")
     except Exception as e:
         logger.error(f"Failed to append {username} to: {filename}. Exception: {e}")
@@ -482,6 +529,7 @@ def init_on_things(source, args, sessions, session_state):
     else:
         stories_percentage = 0
 
+    likes_percentage = get_value(args.likes_percentage, "Chance of liking: {}%", 100)
     follow_percentage = get_value(
         args.follow_percentage, "Chance of following: {}%", 40
     )
@@ -496,6 +544,7 @@ def init_on_things(source, args, sessions, session_state):
     return (
         on_interaction,
         stories_percentage,
+        likes_percentage,
         follow_percentage,
         comment_percentage,
         pm_percentage,
