@@ -2,6 +2,7 @@ import datetime
 import logging
 import re
 from enum import Enum, auto
+from time import sleep
 from colorama import Fore, Style
 from random import choice, randint, uniform
 
@@ -16,7 +17,13 @@ from GramAddict.core.device_facade import (
     Timeout,
 )
 from GramAddict.core.resources import ClassName, ResourceID as resources, TabBarText
-from GramAddict.core.utils import ActionBlockedError, Square, random_sleep, save_crash
+from GramAddict.core.utils import (
+    ActionBlockedError,
+    Square,
+    get_value,
+    random_sleep,
+    save_crash,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -705,15 +712,24 @@ class PostsViewList:
         likes = 0
         if likes_view.exists():
             likes_view_text = likes_view.get_text().replace(",", "")
-            matches = re.search(
+            matches_likes = re.search(
                 r"(?P<likes>\d+) (?:others|likes)", likes_view_text, re.IGNORECASE
             )
-            if hasattr(matches, "group"):
-                likes = int(matches.group("likes"))
+            matches_view = re.search(
+                r"(?P<views>\d+) views", likes_view_text, re.IGNORECASE
+            )
+            if hasattr(matches_likes, "group"):
+                likes = int(matches_likes.group("likes"))
                 logger.info(
                     f"This post has {likes if 'likes' in likes_view_text else likes+1} like(s)."
                 )
                 return likes
+            elif hasattr(matches_view, "group"):
+                views = int(matches_view.group("views"))
+                logger.info(
+                    f"I can see only that this post has {views} views(s). It may contains likes.."
+                )
+                return -1
             else:
                 if likes_view_text.endswith("others"):
                     logger.info("This post has more then 1 like.")
@@ -785,7 +801,7 @@ class PostsViewList:
 
     def _check_if_last_post(self, last_description, current_job):
         """check if that post has been just interacted"""
-        username, is_ad = PostsViewList(self.device)._post_owner(
+        username, is_ad, is_hashtag = PostsViewList(self.device)._post_owner(
             current_job, Owner.GET_NAME
         )
         swiped_a_bit = False
@@ -817,9 +833,9 @@ class PostsViewList:
                     logger.info(
                         "This post has the same description and author as the last one."
                     )
-                    return True, new_description, username, is_ad
+                    return True, new_description, username, is_ad, is_hashtag
                 else:
-                    return False, new_description, username, is_ad
+                    return False, new_description, username, is_ad, is_hashtag
             else:
                 gap_view_obj = self.device.find(resourceIdMatches=ResourceID.GAP_VIEW)
                 feed_composer = self.device.find(
@@ -835,14 +851,14 @@ class PostsViewList:
                         logger.info(
                             "Can't find the description of this post. Maybe it's blank.."
                         )
-                        return False, "", username, is_ad
+                        return False, "", username, is_ad, is_hashtag
 
                 logger.debug(
                     "Can't find the description, try to swipe a little bit down."
                 )
                 UniversalActions(self.device)._swipe_points(direction=Direction.DOWN)
                 swiped_a_bit = True
-        return False, "", username, is_ad
+        return False, "", username, is_ad, is_hashtag
 
     def _if_action_bar_is_over_obj_swipe(self, obj):
         """do a swipe of the amount of the action bar"""
@@ -881,17 +897,30 @@ class PostsViewList:
             UniversalActions(self.device)._reload_page()
 
     def _post_owner(self, current_job, mode: Owner, username=None):
+        """returns a tuple[var, bool, bool]"""
         is_ad = False
+        is_hashtag = False
         if username is None:
             post_owner_obj = self.device.find(
-                resourceIdMatches=(ResourceID.ROW_FEED_PHOTO_PROFILE_NAME)
+                resourceIdMatches=ResourceID.ROW_FEED_PHOTO_PROFILE_NAME
             )
         else:
-            post_owner_obj = self.device.find(
-                resourceIdMatches=(ResourceID.ROW_FEED_PHOTO_PROFILE_NAME),
-                textStartsWith=username,
-            )
+            for _ in range(2):
+                post_owner_obj = self.device.find(
+                    resourceIdMatches=ResourceID.ROW_FEED_PHOTO_PROFILE_NAME,
+                    textStartsWith=username,
+                )
+                notification = self.device.find(
+                    resourceIdMatches=ResourceID.NOTIFICATION_MESSAGE
+                )
+                if not post_owner_obj.exists and notification.exists():
+                    logger.warning(
+                        "There is a notification there! Please disable them in settings.. We will wait 10 seconds before continue.."
+                    )
+                    sleep(10)
+                    continue
         post_owner_clickable = False
+
         for _ in range(2):
             if not post_owner_obj.exists():
                 if mode == Owner.OPEN:
@@ -899,19 +928,19 @@ class PostsViewList:
                         resourceIdMatches=ResourceID.ROW_FEED_COMMENT_TEXTVIEW_LAYOUT,
                         textStartsWith=username,
                     )
-                    for _ in range(2):
-                        if comment_description.exists() is None:
-                            random_sleep()
-                            comment_description = self.device.find(
-                                resourceIdMatches=ResourceID.ROW_FEED_COMMENT_TEXTVIEW_LAYOUT,
-                                textContains=username,
-                            )
-                        else:
-                            break
+                    if (
+                        not comment_description.exists()
+                        and comment_description.count_items() >= 1
+                    ):
+                        comment_description = self.device.find(
+                            resourceIdMatches=ResourceID.ROW_FEED_COMMENT_TEXTVIEW_LAYOUT,
+                            text=comment_description.get_text(),
+                        )
+
                     if comment_description.exists():
                         logger.info("Open post owner from description.")
                         comment_description.child().click()
-                        return True
+                        return True, is_ad, is_hashtag
                 UniversalActions(self.device)._swipe_points(direction=Direction.UP)
                 post_owner_obj = self.device.find(
                     resourceIdMatches=(ResourceID.ROW_FEED_PHOTO_PROFILE_NAME),
@@ -922,22 +951,26 @@ class PostsViewList:
                 break
 
         if not post_owner_clickable:
-            logger.info("Can't find the owner name.")
-            return False, False
+            logger.info("Can't find the owner name, skip.")
+            return False, is_ad, is_hashtag
         if mode == Owner.OPEN:
             logger.info("Open post owner.")
             PostsViewList(self.device)._if_action_bar_is_over_obj_swipe(post_owner_obj)
             post_owner_obj.click()
-            return True
+            return True, is_ad, is_hashtag
         elif mode == Owner.GET_NAME:
             if current_job == "feed":
-                is_ad = PostsViewList(self.device)._check_if_ad(post_owner_obj)
-            return post_owner_obj.get_text().replace("•", "").strip(), is_ad
+                is_ad, is_hashtag, username = PostsViewList(
+                    self.device
+                )._check_if_ad_or_hashtag(post_owner_obj)
+            if username is None:
+                username = post_owner_obj.get_text().replace("•", "").strip()
+            return username, is_ad, is_hashtag
 
         elif mode == Owner.GET_POSITION:
-            return post_owner_obj.get_bounds()
+            return post_owner_obj.get_bounds(), is_ad
         else:
-            return False, False
+            return None, is_ad, is_hashtag
 
     def _get_post_owner_name(self):
         return self.device.find(
@@ -946,6 +979,23 @@ class PostsViewList:
 
     def _like_in_post_view(self, mode: LikeMode):
         post_container = ResourceID.CAROUSEL_MEDIA_GROUP_AND_ZOOMABLE_VIEW_CONTAINER
+        media_obj = self.device.find(resourceIdMatches=post_container)
+        if media_obj.child(resourceIdMatches=ResourceID.CAROUSEL_IMAGE_MEDIA_GROUP):
+            watch_photo_time = get_value(
+                configs.args.watch_photo_time,
+                "Watching photo for {}s.",
+                0,
+                its_time=True,
+            )
+            sleep(watch_photo_time)
+        elif media_obj.child(resourceIdMatches=ResourceID.CAROUSEL_VIDEO_MEDIA_GROUP):
+            watch_video_time = get_value(
+                configs.args.watch_video_time,
+                "Watching video for {}s.",
+                0,
+                its_time=True,
+            )
+            sleep(watch_video_time)
 
         if mode == LikeMode.DOUBLE_CLICK:
             logger.info("Double click photo.")
@@ -984,20 +1034,26 @@ class PostsViewList:
             UniversalActions(self.device)._swipe_points(direction=Direction.DOWN)
             return PostsViewList(self.device)._check_if_liked()
 
-    def _check_if_ad(self, post_owner_obj):
+    def _check_if_ad_or_hashtag(self, post_owner_obj):
         str = "Sponsored"
-        logger.debug("Checking if it's an AD.")
+        is_hashtag = False
+        is_ad = False
+        real_username = None
+        logger.debug("Checking if it's an AD or an hashtag..")
         ad_like_obj = post_owner_obj.sibling(
             resourceId=ResourceID.SECONDARY_LABEL,
         )
+        if post_owner_obj.get_text().startswith("#"):
+            is_hashtag = True
+            logger.debug("Looks like an hashtag, skip.")
         if ad_like_obj.exists():
             if ad_like_obj.get_text() == str:
                 logger.debug("Looks like an AD, skip.")
-                return True
-            else:
-                return False
-        else:
-            return False
+                is_ad = True
+            elif is_hashtag:
+                real_username = ad_like_obj.get_text().split("•")[0].strip()
+
+        return is_ad, is_hashtag, real_username
 
 
 class LanguageView:
@@ -1301,17 +1357,18 @@ class PostsGridView:
             logger.info("It's a IGTV.")
             media_type = MediaType.IGTV
         else:
-            carousel_obj = re.match(
+            carousel_obj = re.finditer(
                 r"((?P<photo>\d+) photo)|((?P<video>\d+) video)",
                 content_desc,
                 re.IGNORECASE,
             )
-            n_photos = (
-                int(carousel_obj.group("photo")) if carousel_obj.group("photo") else 0
-            )
-            n_videos = (
-                int(carousel_obj.group("video")) if carousel_obj.group("video") else 0
-            )
+            n_photos = 0
+            n_videos = 0
+            for match in carousel_obj:
+                if match.group("photo"):
+                    n_photos = int(match.group("photo"))
+                if match.group("video"):
+                    n_videos = int(match.group("video"))
             logger.info(
                 f"It's a carousel with {n_photos} photo(s) and {n_videos} video(s)."
             )
@@ -1606,21 +1663,31 @@ class ProfileView(ActionBarView):
         followers_button = self.device.find(
             resourceIdMatches=ResourceID.ROW_PROFILE_HEADER_FOLLOWERS_CONTAINER
         )
-        if followers_button.exists(Timeout.MEDIUM):
+        if followers_button.exists(Timeout.LONG):
             followers_button.click()
-            return True
+            followers_tab = self.device.find(
+                resourceIdMatches=ResourceID.UNIFIED_FOLLOW_LIST_TAB_LAYOUT
+            ).child(textContains="Followers")
+            if followers_tab.exists(Timeout.LONG):
+                followers_tab.click()
+                return True
         else:
             logger.error("Can't find followers tab!")
             return False
 
     def navigateToFollowing(self):
         logger.info("Navigate to following.")
-        followings_button = self.device.find(
+        following_button = self.device.find(
             resourceIdMatches=ResourceID.ROW_PROFILE_HEADER_FOLLOWING_CONTAINER
         )
-        if followings_button.exists(Timeout.MEDIUM):
-            followings_button.click()
-            return True
+        if following_button.exists(Timeout.LONG):
+            following_button.click()
+            following_tab = self.device.find(
+                resourceIdMatches=ResourceID.UNIFIED_FOLLOW_LIST_TAB_LAYOUT
+            ).child(textContains="Following")
+            if following_tab.exists(Timeout.LONG):
+                following_tab.click()
+                return True
         else:
             logger.error("Can't find following tab!")
             return False
