@@ -11,6 +11,7 @@ from GramAddict.core.storage import FollowingStatus
 from GramAddict.core.utils import random_sleep, save_crash, get_value
 from GramAddict.core.views import (
     FollowingView,
+    ProfileView,
     UniversalActions,
     Direction,
 )
@@ -112,13 +113,7 @@ class ActionUnfollowFollowers(Plugin):
 
         if count <= 0:
             logger.info(
-                "You want to unfollow "
-                + str(count)
-                + ", you have "
-                + str(self.session_state.my_following_count)
-                + " followings, min following is "
-                + str(self.args.min_following)
-                + ". Finish."
+                f"Now you're following {str(self.session_state.my_following_count)} accounts, {'less then' if count <0 else 'equal to'} min following allowed (you set min-following: {str(self.args.min_following)}). No further unfollows are required. Finish."
             )
             return
 
@@ -138,6 +133,7 @@ class ActionUnfollowFollowers(Plugin):
                 storage,
                 self.unfollow_type,
                 self.session_state.my_username,
+                plugin,
             )
             logger.info(
                 f"Unfollowed {self.state.unfollowed_count}, finish.",
@@ -150,7 +146,14 @@ class ActionUnfollowFollowers(Plugin):
             job()
 
     def unfollow(
-        self, device, count, on_unfollow, storage, unfollow_restriction, my_username
+        self,
+        device,
+        count,
+        on_unfollow,
+        storage,
+        unfollow_restriction,
+        my_username,
+        job_name,
     ):
         skipped_list_limit = get_value(self.args.skipped_list_limit, None, 15)
         skipped_fling_limit = get_value(self.args.fling_when_skipped, None, 0)
@@ -159,7 +162,7 @@ class ActionUnfollowFollowers(Plugin):
             skipped_list_limit=skipped_list_limit,
             skipped_fling_limit=skipped_fling_limit,
         )
-        self.open_my_followings(device)
+        ProfileView(device).navigateToFollowing()
         self.iterate_over_followings(
             device,
             count,
@@ -168,20 +171,13 @@ class ActionUnfollowFollowers(Plugin):
             unfollow_restriction,
             my_username,
             posts_end_detector,
+            job_name,
         )
 
     def on_unfollow(self):
         self.state.unfollowed_count += 1
         self.session_state.totalUnfollowed += 1
 
-    def open_my_followings(self, device):
-        logger.info("Open my followings.")
-        followings_button = device.find(
-            resourceIdMatches=self.ResourceID.ROW_PROFILE_HEADER_FOLLOWING_CONTAINER
-        )
-        if followings_button.exists(Timeout.LONG):
-            followings_button.click()
-
     def sort_followings_by_date(self, device, newest_to_oldest=False):
 
         sort_button = device.find(
@@ -219,6 +215,7 @@ class ActionUnfollowFollowers(Plugin):
         unfollow_restriction,
         my_username,
         posts_end_detector,
+        job_name,
     ):
         # Wait until list is rendered
         sorted = False
@@ -331,7 +328,11 @@ class ActionUnfollowFollowers(Plugin):
 
                     if unfollowed:
                         storage.add_interacted_user(
-                            username, self.session_state.id, unfollowed=True
+                            username,
+                            self.session_state.id,
+                            unfollowed=True,
+                            job_name=job_name,
+                            target=username,
                         )
                         on_unfollow()
                         unfollowed_count += 1
@@ -467,12 +468,8 @@ class ActionUnfollowFollowers(Plugin):
         logger.info(
             f"Check if @{username} is following you.", extra={"color": f"{Fore.GREEN}"}
         )
-        following_container = device.find(
-            resourceIdMatches=self.ResourceID.ROW_PROFILE_HEADER_FOLLOWING_CONTAINER
-        )
-        if following_container.exists(Timeout.LONG):
-            following_container.click()
-        else:
+
+        if not ProfileView(device).navigateToFollowing():
             logger.info("Can't load profile in time. Skip.")
             return None
 
